/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Ausrechnet durschschnittliche Tagewerte Die sind aber fuer dem TagesInterwall
 * undefiniert, deswegen erzeugt nur Datensaetze mit nicht ermittelbaren Werten.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AggregatorDTV extends AbstraktAggregator {

	/**
	 * ATG String Konstante.
	 */
	private static final String ATG_LZD_DTV = "atg.langZeitDatenDTV";

	@Override
	public void bearbeiteDatenSatz(final SystemObject objekt,
			final Data datensatz, final long zeitStempel) {
		if (datensatz == null) {
			return;
		}
		final LZZDaten lzz = getLzzDatenForSystemObject(objekt);
		lzz.incrementAnzahlDatenSaetze();
	}

	@Override
	public DataDescription init(final Aspect asp, final ClientDavInterface dav,
			final Collection<SystemObject> objekteLZZ) {

		resetLzzDaten();

		for (final SystemObject lzz : objekteLZZ) {
			final LZZDaten lzzStruct = new LZZDaten();
			final Data data = dav.createData(dav.getDataModel()
					.getAttributeGroup(AggregatorDTV.ATG_LZD_DTV));
			lzzStruct.setErgebnissDatenSatz(data);
			setLeer(data);
			addLzzDaten(lzz, lzzStruct);
		}

		return new DataDescription(dav.getDataModel().getAttributeGroup(
				AggregatorDTV.ATG_LZD_DTV), asp);
	}

	@Override
	public void erzeugeErgebniss(final SystemObject objekt,
			final long zeitStempelVon, final long zeitStempelBis,
			final boolean ermittelbar) {

		final LZZDaten lzz = getLzzDatenForSystemObject(objekt);
		final Data ergebnissDatenSatz = lzz.getErgebnissDatenSatz();

		if (lzz.getZeitStempelLetztesDS() >= zeitStempelVon) {
			return;
		}

		final ResultData result = new ResultData(objekt, getDdAggregierteLzd(),
				zeitStempelVon, ergebnissDatenSatz, true);
		sendeDatenSatz(objekt, result);
		lzz.setAnzahlDatenSaetze(0);
		lzz.setZeitStempelLetztesDS(zeitStempelVon);
	}

}
