/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;

/**
 * Ein Aggregator, der die Historie zu speichern muss um seine Werte zu
 * berechnen koennen - Geschwindigkeit85 usw.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public abstract class AggregatorMitHistorie extends AbstraktAggregator {

	/**
	 * Erweitert die einfache Struktur mit LZZ aggregierten Datensatz um nicht
	 * aggregierte LZD - historische Werte.
	 *
	 */
	protected class LZZHistDaten extends LZZDaten {
		/**
		 * Liste der DatenSaetze aus der Historie, die zur Berechnung der Werte
		 * benutzt werden.
		 */
		private final List<Data> datenSatzHistorie = new LinkedList<>();

		/**
		 * Fuegt ein neues DatenSatz in die Historie ein.
		 *
		 * @param data
		 *            die hinzuzufügenden Daten
		 */
		public void addDatenSatzZurHistorie(final Data data) {
			datenSatzHistorie.add(data);
		}

		/**
		 * Loescht die Historie.
		 */
		public void clearHistorie() {
			datenSatzHistorie.clear();
		}

		/**
		 * Ergibt alle Datensaetze aus der Historie.
		 *
		 * @return Liste der DatenSaetze
		 */
		public List<Data> getHistorie() {
			return datenSatzHistorie;
		}
	}

	/**
	 * Ermoeglicht die Datensaetze zu sortiernn Der parameter im Konstruktor ist
	 * der Parameter, nach dem die Vergleichung geht.
	 *
	 * @author bachraty
	 */
	protected class DatenSatzVergleicher implements Comparator<Data> {

		private final String attribut;

		/**
		 * Konsturktor.
		 *
		 * @param attrib
		 *            Der Atirbut des DatenSatzes der zur Vergleichung benutzt
		 *            wird
		 */
		public DatenSatzVergleicher(final String attrib) {
			attribut = attrib;
		}

		@Override
		public int compare(final Data arg0, final Data arg1) {
			final Data subData0 = arg0.getItem(attribut);
			final Data subData1 = arg1.getItem(attribut);
			final int f0 = subData0.getUnscaledValue(AbstraktAggregator.STR_WERT).intValue();
			final int f1 = subData1.getUnscaledValue(AbstraktAggregator.STR_WERT).intValue();
			return f0 - f1;
		}

	}
}
