/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.dua.schnittstellen.IObjektWeckerListener;

/**
 * Erweitert die Klasse DatenEndgeraet um Statusinformationen und andere Daten,
 * die einen System Daten Eingeraet beschreiben
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class SystemDatenEndgeraet extends DatenEndgeraet implements
		IObjektWeckerListener {
	/**
	 * Status eines System DE
	 */
	public enum SystemDeStatus {
		/**
		 * Das Endgeraet kann man nicht erreichen
		 */
		DE_STAT_NICHT_VERBUNDEN,
		/**
		 * Die Daten vom Eedngaeraet sind nicht zur Archvivierung parametriert
		 */
		DE_STAT_NICHT_PARAMETRIERT,
		/**
		 * Es laeuft gerade eine Abfrage
		 */
		DE_STAT_ABFRAGE_LAEUFT,
		/**
		 * Das Datenendgeraet ist vorberietet fuer die Abfrage
		 */
		DE_STAT_ABFRAGE_MOEGLICH
	}

	/**
	 * Der Status einse SystemDe ist am Anfang "nicht verbunden"
	 */
	private SystemDeStatus status = SystemDeStatus.DE_STAT_NICHT_VERBUNDEN;
	/**
	 * Zeit des letzten Abrufs des DEs
	 */
	private long letzterAbruf = AbfrPufferKonstanten.ZEIT_BEGINN;

	/**
	 * Standardkonstruktor
	 *
	 * @param de
	 *            SystemObjekt DatenEndgeraet
	 * @param station
	 *            Station zu die das DE gehoert
	 */
	public SystemDatenEndgeraet(final SystemObject de, final Station station) {
		super(de, station);
	}

	@Override
	public String toString() {
		return String.format("%s STATUS:%s\n", getSoDE().getPid(), this.status);
	}

	/**
	 * Liefert dem Status des System DEs
	 *
	 * @return Status
	 */
	public SystemDeStatus getStatus() {
		return status;
	}

	/**
	 * Setzt dem Status des System DEs
	 *
	 * @param status
	 *            the status to set
	 */
	public void setStatus(final SystemDeStatus status) {
		this.status = status;
	}

	/**
	 * Liefert dem Zeitstemepel des leztes Abrufs
	 *
	 * @return the letzterAbruf
	 */
	public long getLetzterAbruf() {
		return letzterAbruf;
	}

	/**
	 * Setzt dem Zeitstempel des letztes Abrufs
	 *
	 * @param letzterAbruf
	 *            Zeitstempel
	 */
	public void setLetzterAbruf(final long letzterAbruf) {
		this.letzterAbruf = letzterAbruf;
	}

	@Override
	public void alarm() {
		erwarteKeineAntwort(false);
	}

	/**
	 * Wenn ein Timeout oder eine Negative Quittung kommt, muss man die
	 * Erwartungen von der Abfrage anullieren
	 *
	 * @param negQuittung
	 *            true, wenn es sich um eine Negative Quittung handelt
	 */
	public void erwarteKeineAntwort(final boolean negQuittung) {

		final Station station = getStation();
		synchronized (station.getMutex()) {

			if (status == SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT) {
				status = SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH;
				Collection<DatenEndgeraet> liste = new LinkedList<>();
				if (station.getSystemLveDe() == this) {
					liste = station.getLveDeListe();
				} else if (station.getSystemAxlDe() == this) {
					liste = station.getAxlDeListe();
				}

				for (final DatenEndgeraet de : liste) {
					de.setAnzahlErwarteteDatenSaetze(0);
				}
				this.letzterAbruf = AbfrPufferKonstanten.ZEIT_BEGINN;
			}
		}
		// Ermittelt der Station dass der Timeout gekommen ist,
		// Abfrage ist beendet,
		if (!negQuittung) {
			station.abfrageEnde(true);
		}
	}
}